#
# Copyright (c) 2013 Christophe Varoqui <christophe.varoqui@opensvc.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import os
import stat
import pwd
import grp

from rcGlobalEnv import rcEnv
import rcStatus
import rcExceptions as ex
from resources import Resource

class File(Resource):
    def check_perm(self, f):
        if 'perm' not in f:
            return
        try:
            mode = oct(stat.S_IMODE(os.stat(f['dst']).st_mode))
        except:
            self.issues.append('can not stat file %s'%f['dst'])
            return
        mode = str(mode).lstrip("0")
        if mode != str(f['perm']):
            self.issues.append('%s mode should be %s but is %s'%(f['dst'], f['perm'], mode))
            return

    def get_uid(self, f):
        try:
            info = pwd.getpwnam(f['user'])
            return info[2]
        except:
            err = "user %s does not exist"%f['user']
            if err in self.issues:
                return
            self.issues.append(err)

    def get_gid(self, f):
        try:
            info = grp.getgrnam(f['group'])
            return info[2]
        except:
            err = "group %s does not exist"%f['group']
            if err in self.issues:
                return
            self.issues.append(err)

    def check_uid(self, f):
        if len(f['user']) == 0:
            return
        tuid = self.get_uid(f)
        uid = os.stat(f['dst']).st_uid
        if uid is None:
            return
        if uid != tuid:
            self.issues.append('%s uid should be %s but is %s'%(f['dst'], tuid, str(uid)))

    def check_gid(self, f):
        if len(f['group']) == 0:
            return
        tgid = self.get_gid(f)
        gid = os.stat(f['dst']).st_gid
        if gid is None:
            return
        if gid != tgid:
            self.issues.append('%s gid should be %s but is %s'%(f['dst'], tgid, str(gid)))

    def is_up_one(self, f):
        if not os.path.exists(f['src']):
            self.issues.append("%s reference file is not present"%f['src'])
        if not os.path.exists(f['dst']):
            self.issues.append("%s is not installed"%f['dst'])
        self.check_perm(f)
        self.check_uid(f)
        self.check_gid(f)

    def is_up(self):
        self.issues = []
        for f in self.files:
            self.is_up_one(f)
        if len(self.issues) > 0:
            return False
        return True

    def start(self):
        if self.is_up():
            self.log.info("%s is already up" % self.path)
            return
        self.can_rollback = True

    def stop(self):
        if not self.is_up():
            self.log.info("%s is already down" % self.path)
            return 0

    def _status(self, verbose=False):
        try:
            up = self.is_up()
        except ex.excError as e:
            self.status_log(str(e))
            return rcStatus.WARN
        if len(self.issues) > 0:
            self.status_log('.\n'.join(self.issues))
            return rcStatus.WARN
        if rcEnv.nodename in self.always_on:
            if up: return rcStatus.STDBY_UP
            else: return rcStatus.STDBY_DOWN
        else:
            if up: return rcStatus.UP
            else: return rcStatus.DOWN

    def parse_entry(self, e):
        if len(e) == 0:
            return
        l = e.split(':')
        if len(l) != 5:
            raise ex.excError("malformed file definition '%s'. expected format is user:group:perm:src:dst"%e)
        data = {
          'user': l[0],
          'group': l[1],
          'perm': l[2],
          'src': l[3],
          'dst': l[4],
        }
        return data

    def __init__(self, rid, files, always_on=set([]),
                 disabled=False, tags=set([]), optional=False, monitor=False):
        Resource.__init__(self, rid, type="file", always_on=always_on,
                          disabled=disabled, tags=tags, optional=optional,
                          monitor=monitor)
        print files
        l = files.replace('\\', '').split(',')
        self.files = []
        for e in l:
            data = self.parse_entry(e)
            if data is None:
                continue
            self.files.append(data)

        n = len(self.files)
        if n == 0:
             self.label = "no files"
        elif n == 1:
             self.label = "%d file" % n
        else:
             self.label = "%d files" % n
